# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Alert'
        db.create_table('alert_alert', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('backend', self.gf('django.db.models.fields.CharField')(default='EmailBackend', max_length=20)),
            ('alert_type', self.gf('django.db.models.fields.CharField')(max_length=25)),
            ('title', self.gf('django.db.models.fields.CharField')(default=u'Premium Domain Finder alert', max_length=250)),
            ('body', self.gf('django.db.models.fields.TextField')()),
            ('when', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('last_attempt', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('is_sent', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('failed', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('alert', ['Alert'])

        # Adding model 'AlertPreference'
        db.create_table('alert_alertpreference', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('alert_type', self.gf('django.db.models.fields.CharField')(max_length=25)),
            ('backend', self.gf('django.db.models.fields.CharField')(max_length=25)),
            ('preference', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('alert', ['AlertPreference'])

        # Adding unique constraint on 'AlertPreference', fields ['user', 'alert_type', 'backend']
        db.create_unique('alert_alertpreference', ['user_id', 'alert_type', 'backend'])


    def backwards(self, orm):
        
        # Removing unique constraint on 'AlertPreference', fields ['user', 'alert_type', 'backend']
        db.delete_unique('alert_alertpreference', ['user_id', 'alert_type', 'backend'])

        # Deleting model 'Alert'
        db.delete_table('alert_alert')

        # Deleting model 'AlertPreference'
        db.delete_table('alert_alertpreference')


    models = {
        'alert.alert': {
            'Meta': {'object_name': 'Alert'},
            'alert_type': ('django.db.models.fields.CharField', [], {'max_length': '25'}),
            'backend': ('django.db.models.fields.CharField', [], {'default': "'EmailBackend'", 'max_length': '20'}),
            'body': ('django.db.models.fields.TextField', [], {}),
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'failed': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_sent': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_attempt': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'default': "u'Premium Domain Finder alert'", 'max_length': '250'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"}),
            'when': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'})
        },
        'alert.alertpreference': {
            'Meta': {'unique_together': "(('user', 'alert_type', 'backend'),)", 'object_name': 'AlertPreference'},
            'alert_type': ('django.db.models.fields.CharField', [], {'max_length': '25'}),
            'backend': ('django.db.models.fields.CharField', [], {'max_length': '25'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'preference': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['alert']



from alert.utils import BaseAlert
from alert.signals import admin_alert_saved

class DjangoAdminAlert(BaseAlert):
    title = 'Admin Alert (created in Django Admin Interface)'
    description = "Send alerts directly to the site's users from the django admin"
    
    signal = admin_alert_saved
    template_filetype = 'html'
    
    # by default all users will receive this alert
    default = True
    
    def get_applicable_users(self, instance, recipients, **kwargs):
        return recipients

    def get_send_time(self, instance, **kwargs):
        return instance.send_at


from django.conf import settings
from django.utils import timezone
from django.contrib.auth.models import Group
from django.contrib.auth.models import User as OriginalUser
from django.contrib.sites.models import Site
from django.db import models

from alert.utils import ALERT_TYPE_CHOICES, ALERT_BACKEND_CHOICES, ALERT_TYPES, ALERT_BACKENDS
from alert.managers import AlertManager, PendingAlertManager, AlertPrefsManager
from alert.exceptions import CouldNotSendError
from alert.signals import alert_sent


def get_alert_default_title():
    return "%s alert" % Site.objects.get_current().name

def get_alert_default_site():
    return Site.objects.get_current().id


class Alert(models.Model):
    user = models.ForeignKey(getattr(settings, 'AUTH_USER_MODEL', OriginalUser))
    backend = models.CharField(max_length=20, default='EmailBackend', choices=ALERT_BACKEND_CHOICES)
    alert_type = models.CharField(max_length=25, choices=ALERT_TYPE_CHOICES)

    title = models.CharField(max_length=250, default=get_alert_default_title)
    body = models.TextField()

    when = models.DateTimeField(default=timezone.now)
    created = models.DateTimeField(default=timezone.now)
    last_attempt = models.DateTimeField(blank=True, null=True)
    
    is_sent = models.BooleanField(default=False)
    failed = models.BooleanField(default=False)

    site = models.ForeignKey(Site, default=get_alert_default_site)

    objects = AlertManager()
    pending = PendingAlertManager()
    
    
    def send(self, commit=True):
        backend = self.backend_obj
        try:
            backend.send(self)
            self.is_sent = True
            self.failed = False
            alert_sent.send(sender=self.alert_type_obj, alert=self)
            
        except CouldNotSendError:
            self.failed = True
        
        self.last_attempt = timezone.now()
        if commit:
          self.save()
    
    @property
    def alert_type_obj(self):
        return ALERT_TYPES[self.alert_type]
    
    @property
    def backend_obj(self):
        return ALERT_BACKENDS[self.backend]



class AlertPreference(models.Model):
    user = models.ForeignKey(getattr(settings, 'AUTH_USER_MODEL', OriginalUser))
    alert_type = models.CharField(max_length=25, choices=ALERT_TYPE_CHOICES)
    backend = models.CharField(max_length=25, choices=ALERT_BACKEND_CHOICES)
    
    preference = models.BooleanField(default=False)
    
    objects = AlertPrefsManager()
    
    class Meta:
        unique_together = ('user', 'alert_type', 'backend')
        
    @property
    def alert_type_obj(self):
        return ALERT_TYPES[self.alert_type]
    
    @property
    def backend_obj(self):
        return ALERT_BACKENDS[self.backend]
    
    
    
class AdminAlert(models.Model):
    title = models.CharField(max_length=250)
    body = models.TextField()
    
    recipients = models.ForeignKey(Group, null=True, help_text="who should receive this message?")

    send_at = models.DateTimeField(default=timezone.now, help_text="schedule the sending of this message in the future")
    draft = models.BooleanField(default=False, verbose_name="Save as draft (don't send/queue now)")
    sent = models.BooleanField(default=False)
    
    
   
import alert.listeners #@UnusedImport
import alert.backends #@UnusedImport
import alert.alerts #@UnusedImport


import re

from django.core.mail import send_mail
from django.core.mail import EmailMultiAlternatives
from django.conf import settings

from alert.utils import BaseAlertBackend
from alert.exceptions import CouldNotSendError
from django.template.defaultfilters import striptags

strip_head = re.compile(r"<head.*?</head>", re.DOTALL)
strip_style = re.compile(r"<style.*?</style>", re.DOTALL)
strip_script = re.compile(r"<script.*?</script>", re.DOTALL)
gt_2_lines_whitespace = re.compile(r"\s*\n\s*\n\s*", re.DOTALL)
multiple_spaces = re.compile(r"[\f\v\t ]+", re.DOTALL)

link1 = re.compile(r"(<a.*?href=\"([^\"]*)\".*?</a>)", re.I)
link2 = re.compile(r"(<a.*?href='([^']*)'.*?</a>)", re.I)
link_replace = lambda m: "%s (%s)" % m.groups()

def to_text(html_content):
    html_content = strip_style.sub("", html_content)
    html_content = strip_script.sub("", html_content)
    html_content = strip_head.sub("", html_content)
    
    html_content = link1.sub(link_replace, html_content)
    html_content = link2.sub(link_replace, html_content)
    
    html_content = striptags(html_content)
    
    # max out at 2 empty lines
    html_content = gt_2_lines_whitespace.sub("\n\n", html_content)
    html_content = multiple_spaces.sub(" ", html_content)
    
    return html_content


class EmailBackend(BaseAlertBackend):
    title = "Email"
    
    def send(self, alert):
        recipient = alert.user.email
        if not recipient: raise CouldNotSendError
        
        subject = alert.title.replace("\n", "").strip()
        to = [recipient]
        from_email = settings.DEFAULT_FROM_EMAIL 
        
        try:
            if alert.alert_type_obj.template_filetype == 'html':
                html_content = alert.body
                text_content = to_text(html_content)
                
                msg = EmailMultiAlternatives(subject, text_content, from_email, to)
                msg.attach_alternative(html_content, "text/html")
                msg.send()
                
            else:
                send_mail(subject, alert.body, from_email, to)
        except Exception, e:
            print "sending to", recipient, "failed"
            print e
            print "\n"
            raise CouldNotSendError
        
        



from collections import defaultdict
from django.db.models import Manager
from django.db.models.query import QuerySet
from django.utils import timezone
from alert.utils import ALERT_TYPES, ALERT_BACKENDS


class AlertManager(Manager):
    pass


class PendingAlertManager(AlertManager):
    """
    Alerts that are ready to send NOW.
    
    This is not the same as unsent alerts; alerts scheduled to be sent in the
    future will not be affected by this manager.
    """
    def get_query_set(self, *args, **kwargs):
        qs = super(PendingAlertManager, self).get_query_set(*args, **kwargs)
        return qs.filter(when__lte=timezone.now(), is_sent=False)
    
    def get_queryset(self, *args, **kwargs):
        qs = super(PendingAlertManager, self).get_queryset(*args, **kwargs)
        return qs.filter(when__lte=timezone.now(), is_sent=False)


class AlertPrefsManager(Manager):
    def get_queryset_compat(self, *args, **kwargs):
        getqs = self.get_queryset if hasattr(Manager, "get_queryset") else self.get_query_set
        return getqs(*args, **kwargs)
    
    def get_user_prefs(self, user):
        if not user.is_authenticated():
            return dict(((notice_type.id, backend.id), False)
                            for notice_type in ALERT_TYPES.values()
                            for backend in ALERT_BACKENDS.values()
                        )
        
            
        alert_prefs = self.get_queryset_compat().filter(user=user)
        
        prefs = {}
        for pref in alert_prefs:
            prefs[pref.alert_type, pref.backend] = pref.preference
        
        for notice_type in ALERT_TYPES.values():
            for backend in ALERT_BACKENDS.values():
                if (notice_type.id, backend.id) not in prefs:
                    default_pref = notice_type.get_default(backend.id)
                    prefs[notice_type.id, backend.id] = default_pref
        return prefs
    
                
    def get_recipients_for_notice(self, notice_type, users):
        if isinstance(notice_type, basestring):
            notice_type = ALERT_TYPES[notice_type]
        
        if not users: return ()
        
        # this optimization really shouldn't be necessary, but it makes a huge difference on mysql
        if isinstance(users, QuerySet):
          user_ids = list(users.values_list("id", flat=True))
        else:
          user_ids = [u.id for u in users]
        
        alert_prefs = self.get_queryset_compat().filter(alert_type=notice_type.id).filter(user__in=user_ids)
        
        prefs = {}
        for pref in alert_prefs:
            prefs[pref.user_id, pref.backend] = pref.preference
        
        user_cache = {}
        for user in users:
            user_cache[user.id] = user
            for backend in ALERT_BACKENDS.values():
                if (user.id, backend.id) not in prefs:
                    prefs[user.id, backend.id] = notice_type.get_default(backend.id)
        
        return ((user_cache[user_id], ALERT_BACKENDS[backend_id]) for ((user_id, backend_id), pref) in prefs.items() if pref)
    



from django.contrib import admin
from django.utils import timezone
from alert.models import Alert, AlertPreference, AdminAlert
from alert.signals import admin_alert_saved


class AlertAdmin(admin.ModelAdmin):
    list_display = ('username', 'title', 'backend', 'alert_type', 'failed', 'is_sent', 'created',)
    list_filter = ('alert_type', 'backend', 'is_sent', 'failed')
    search_fields = ('=user__username', '=user__email')
    actions = ['resend']
    raw_id_fields = ("user",)
    
    def queryset(self, request):
        qs = super(AlertAdmin, self).queryset(request)
        if hasattr(qs, "prefetch_related"):
            qs = qs.prefetch_related("user")
        return qs
    
    def username(self, obj):
        return obj.user.username
    
    def resend(self, request, qs):
        for alert in qs:
            alert.send()
    resend.short_description = "Resend selected alerts"



class AlertPrefAdmin(admin.ModelAdmin):
    list_display = ("user", 'alert_type', "backend", 'preference')
    list_filter = ('alert_type', 'backend', 'preference')
    search_fields = ('=user__username', '=user__email')
    actions = ['subscribe', 'unsubscribe']
    raw_id_fields = ("user",)
    
    
    def unsubscribe(self, request, qs): 
        qs.update(preference=False)
    unsubscribe.short_description = 'Set selected preferences to "Unsubscribed"'
    
    
    def subscribe(self, request, qs): 
        qs.update(preference=True)
    subscribe.short_description = 'Set selected preferences to "Subscribed"'



class AdminAlertAdmin(admin.ModelAdmin):
    list_display = ("title", "status", "send_time",)
    search_fields = ("title",)
    
    fieldsets = (
        (None, {
            'fields': ('title', 'body',)
        }),
        ("Recipients", {
#            'classes': ('collapse',),
            'fields': ('recipients',) 
        }),
        ("Advanced", {
            'classes': ('collapse',),
            'fields': ('send_at', 'draft')
        }),
    )
    
    
    def get_readonly_fields(self, request, obj=None):
        # don't allow editing if it's already sent
        if obj and obj.sent:
            return ("title", 'body', 'recipients', 'send_at', 'draft')
        else: 
            return ()
        
        
    def save_model(self, request, obj, form, change):
        is_draft = obj.draft
        if is_draft:
            # set the draft property false for next time
            obj.draft = False
        
        # if it's already been sent then that's it
        obj.sent = obj.sent or not is_draft
        
        obj.save()
        
        # for now, sent to all site users
        recipients = obj.recipients.user_set.all()
        
        if not is_draft:
            admin_alert_saved.send(sender=AdminAlert, instance=obj, recipients=recipients)
        
        
    def status(self, obj):
        if obj.sent:
            return "scheduled" if obj.send_at < timezone.now() else "sent"
        else:
            return "unsent (saved as draft)"
        

    def send_time(self, obj):
        return "-" if not obj.sent else obj.send_at
        
    
    

admin.site.register(Alert, AlertAdmin)
admin.site.register(AlertPreference, AlertPrefAdmin)
admin.site.register(AdminAlert, AdminAlertAdmin)



from alert.exceptions import AlertIDAlreadyInUse, AlertBackendIDAlreadyInUse,\
    InvalidApplicableUsers
import django
from django.conf import settings
from django.utils import timezone
from django.template.loader import render_to_string, get_template
from django.contrib.sites.models import Site
from django.template import TemplateDoesNotExist
from django.db import models
from itertools import islice

from alert.compat import get_user_model

ALERT_TYPES = {}
ALERT_BACKENDS = {}

ALERT_TYPE_CHOICES = [] 
ALERT_BACKEND_CHOICES = []

def grouper(n, iterable):
    iterable = iter(iterable)
    while True:
        chunk = tuple(islice(iterable, n))
        if not chunk: return
        yield chunk

class AlertMeta(type):

    def __new__(cls, name, bases, attrs):
        new_alert = super(AlertMeta, cls).__new__(cls, name, bases, attrs)
        
        # If this isn't a subclass of BaseAlert, don't do anything special.
        parents = [b for b in bases if isinstance(b, AlertMeta)]
        if not parents: 
            return new_alert
        
        # allow subclasses to use the auto id feature
        id = getattr(new_alert, 'id', name)
        for parent in parents:
            if getattr(parent, 'id', None) == id:
                id = name
                break
        
        new_alert.id = id
        
        if new_alert.id in ALERT_TYPES.keys():
            raise AlertIDAlreadyInUse("The alert ID, \"%s\" was delared more than once" % new_alert.id)
        
        ALERT_TYPES[new_alert.id] = new_alert()
        ALERT_TYPE_CHOICES.append((new_alert.id, new_alert.title))
        
        return new_alert



class BaseAlert(object):
    __metaclass__ = AlertMeta
    
    default = False
    sender = None
    template_filetype = "txt"
    
    
    
    def __init__(self):
        kwargs = {}
        if self.sender:
            kwargs['sender'] = self.sender
        
        self.signal.connect(self.signal_handler, **kwargs)
    
    def __repr__(self):
        return "<Alert: %s>" % self.id
    
    def __str__(self):
        return str(self.id)
    
    def signal_handler(self, **kwargs):
    
        if self.before(**kwargs) is False: 
            return

        from alert.models import AlertPreference
        from alert.models import Alert
        
        users = self.get_applicable_users(**kwargs)
        if isinstance(users, models.Model):
            users = [users]
        
        try:
            user_count = users.count()
        except:
            user_count = len(users)
            
        User = get_user_model()
        if user_count and not isinstance(users[0], User):
            raise InvalidApplicableUsers("%s.get_applicable_users() returned an invalid value. Acceptable values are a django.contrib.auth.models.User instance OR an iterable containing 0 or more User instances" % (self.id))
        
        site = Site.objects.get_current()
        
        def mk_alert(user, backend):
            context = self.get_template_context(BACKEND=backend, USER=user, SITE=site, ALERT=self, **kwargs)
            template_kwargs = {'backend': backend, 'context': context }
            return Alert(
                          user=user, 
                          backend=backend.id,
                          alert_type=self.id,
                          when=self.get_send_time(**kwargs),
                          title=self.get_title(**template_kwargs),
                          body=self.get_body(**template_kwargs)
                          )
        alerts = (mk_alert(user, backend) for (user, backend) in AlertPreference.objects.get_recipients_for_notice(self.id, users))
        
        # bulk create is much faster so use it when available
        if django.VERSION >= (1, 4) and getattr(settings, 'ALERT_USE_BULK_CREATE', True):
            created = 0
            for alerts_group in grouper(100, alerts):
                # break bulk create into groups of 100 to avoid the dreaded
                # OperationalError: (2006, 'MySQL server has gone away')
                Alert.objects.bulk_create(alerts_group)
                created += 100
        else:
            for alert in alerts: alert.save()
    
    
    def before(self, **kwargs):
        pass
    
    
    def get_send_time(self, **kwargs):
        return timezone.now()


    def get_applicable_users(self, instance, **kwargs):
        return [instance.user]
    
    
    def get_template_context(self, **kwargs):
        return kwargs


    def _get_template(self, backend, part, filetype='txt'):
        template = "alerts/%s/%s/%s.%s" % (self.id, backend.id, part, filetype)
        try:
            get_template(template)
            return template
        except TemplateDoesNotExist:
            pass
        
        template = "alerts/%s/%s.%s" % (self.id, part, filetype)
        get_template(template)
        
        return template
        
    
    def get_title_template(self, backend, context):
        return self._get_template(backend, 'title', self.template_filetype)
    
    
    def get_body_template(self, backend, context):
        return self._get_template(backend, 'body', self.template_filetype)
    
    
    def get_title(self, backend, context):
        template = self.get_title_template(backend, context)
        return render_to_string(template, context)
    
    
    def get_body(self, backend, context):
        template = self.get_body_template(backend, context)
        return render_to_string(template, context)
    
    
    def get_default(self, backend):
        if isinstance(self.default, bool): 
            return self.default
        return self.default[backend]
    
    

class AlertBackendMeta(type):

    def __new__(cls, name, bases, attrs):
        new_alert_backend = super(AlertBackendMeta, cls).__new__(cls, name, bases, attrs)
        
        # If this isn't a subclass of BaseAlert, don't do anything special.
        parents = [b for b in bases if isinstance(b, AlertBackendMeta)]
        if not parents: 
            return new_alert_backend
        
        new_alert_backend.id = getattr(new_alert_backend, 'id', name)
        
        if new_alert_backend.id in ALERT_BACKENDS.keys(): 
            raise AlertBackendIDAlreadyInUse("The alert ID, \"%s\" was delared more than once" % new_alert_backend.id)
        
        ALERT_BACKENDS[new_alert_backend.id] = new_alert_backend()
        ALERT_BACKEND_CHOICES.append((new_alert_backend.id, new_alert_backend.title))
        
        return new_alert_backend



class BaseAlertBackend(object):
    __metaclass__ = AlertBackendMeta
    
    def __repr__(self):
        return "<AlertBackend: %s>" % self.id
    
    def __str__(self):
        return str(self.id)
    
    def mass_send(self, alerts):
        from .models import Alert
        if isinstance(alerts, Alert):
            self.send(alerts)
        else:
            [self.send(alert) for alert in alerts]
              
              
def super_accepter(arg, lookup_dict):
    """  
    for the alerts and backends keyword arguments... 
      - provides resonable defaults
      - accept a single alert/backend or a list of them
      - accept alert/backend class or the a string containing the alert/backend id
    """
    # reasonable default
    if arg is None: return lookup_dict.values()
    
    # single item or a list
    if not isinstance(arg, (tuple, list)): 
        arg = [arg]
        
    # normalize the arguments
    ids = ((a if isinstance(a, basestring) else a.id) for a in arg)
    
    # remove duplicates
    _set = {}
    ids = (_set.setdefault(id,id) for id in ids if id not in _set)
    
    # lookup the objects 
    return [lookup_dict[id] for id in ids]


def unsubscribe_user(user, alerts=None, backends=None):
    from .forms import UnsubscribeForm 
    form = UnsubscribeForm(user=user, alerts=alerts, backends=backends)
    
    data = dict((field, False) for field in form.fields.keys())
    
    form = UnsubscribeForm(data, user=user, alerts=alerts, backends=backends)
    assert(form.is_valid())
    form.save()



import django.dispatch

alert_sent = django.dispatch.Signal(providing_args=['alert'])
preference_updated = django.dispatch.Signal(providing_args=['user', 'preference', 'instance'])
admin_alert_saved = django.dispatch.Signal(providing_args=['instance', 'recipients'])


from setuptools import setup

setup(
    name='django-alert',
    version='0.8.1',
    
    author='James Robert',
    author_email='jiaaro@gmail.com',
    
    description=('Send alerts, notifications, and messages based '
                'on events in your django application'),
    long_description=open('README.markdown').read(),
    
    license='MIT',
    keywords='django alerts notifications social',
    
    url='https://djangoalert.com',
    
    install_requires=[
        "django >= 1.4",
    ],
    
    packages=[
        'alert', 
        'alert.management', 
        'alert.management.commands', 
        'alert.migrations',
    ],
    
    include_package_data=True,
    
    classifiers=[
    	'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Framework :: Django',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Utilities'
    ]
)
